{ open Parser}

rule token = parse
  [' ' '\r' '\n' '\t']	{ token lexbuf }   (* Whitespace *)
| "/*"                  { comment lexbuf } (* Comments *)
| '('                   { LPAREN }
| ')'                   { RPAREN }
| '{'                   { LBRACE }
| '}'                   { RBRACE }
| '['			{ LBRACK }		
| ']'			{ RBRACK }
| ';'                   { SEMI }
| ':'                   { COLON } 
| '+'                   { PLUS }
| '-'                   { MINUS }
| '*'                   { TIMES }
| '/'                   { DIVIDE }
| ":="                  { ASSIGN }
| ","                   { COMMA }
| "->"                  { ARROW }
| "="                   { EQ }
| "!="                  { NEQ }
| '<'                   { LT }
| "<="                  { LEQ }
| ">"                   { GT }
| ">="                  { GEQ }
| "%"                   { MOD }
| "if"                  { IF } 
| "then"                { THEN }
| "else"                { ELSE }
| "for"                 { FOR }
| "foreach"             { FOREACH }
| "return"              { RETURN }
| '.'			{ DOT }
| "Input"               { CSV_INPUT }      (* CSV Input Array *)
| "Int"                 { INT_TYPE }       (* Integer Type *)
| "String"              { STRING_TYPE }    (* String Type *) 
| "Boolean"             { BOOLEAN_TYPE }   (* Boolean Type *)
| "Float"               { FLOAT_TYPE }     (* Float Type *)
| "Object"              { OBJECT_DEF }     (* Object Def *)
| "Obj"                 { OBJECT_TYPE }    (* Object Type *)
| "Table"               { TABLE_DEF }      (* Table Def *)
| "Tab"                 { TABLE_TYPE }     (* Table Type *)
| "CSV"                 { CSV_DEF }        (* CSV Def *)
| "Func"                { FUNC }           (* Function *)
| "true" | "false" as lit { BOOLLIT(bool_of_string lit) }  (* True/False Value *)
| eof			{ EOF }		   (* End Of File *)
|  ('-')?(['0'-'9']+)"."(['0'-'9']+)?(['e''E']('+'|'-')?['0'-'9']+)?('F''f''L''l')?  |
   ('-')?"."(['0'-'9']+)(['e''E']('+'|'-')?['0'-'9']+)?('F''f''L''l')?               |        
   ('-')?(['0'-'9']+)"."?['e''E']('+'|'-')?(['0'-'9']+)('F''f''L''l')? as num    
      { FLOAT( float_of_string num) } 
| ('-')?['0'-'9']+ as lit     { LITERAL(int_of_string lit) } 
| ['a'-'z' 'A'-'Z' '_']['a'-'z' 'A'-'Z' '0'-'9' '_']* as id { ID(id) }
| '"' [^'"']* '"' as strlit   { STRING_LITERAL(strlit) } 
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/"                  { token lexbuf }
  | _                   { comment lexbuf }
